//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "HPGLtoDXF.h"
#include "StringHelp.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

/*
HPGL-1 to DXF translator
A default size of 0.025mm is assumed for each unit in the HPGL file, so the
DXF file is in mm.

Copyright R.W.Williams
Last amended 27 March 2007
*/

THPGLtoDXFForm *HPGLtoDXFForm;
//---------------------------------------------------------------------------
__fastcall THPGLtoDXFForm::THPGLtoDXFForm(TComponent* Owner)
    : TForm(Owner)
{
    Colour = "7";

}
//---------------------------------------------------------------------------
void __fastcall THPGLtoDXFForm::BrowseButton1Click(TObject *Sender)
{
    if (OpenDialog1->Execute()) {
        Edit1->Text = OpenDialog1->FileName;
        OpenDialog1->InitialDir = ExtractFilePath(Edit1->Text);
        Edit2->Text = ChangeFileExt(Edit1->Text, ".dxf");
        SaveDialog1->InitialDir = ExtractFilePath(Edit1->Text);
        }
    
}
//---------------------------------------------------------------------------
void __fastcall THPGLtoDXFForm::BrowseButton2Click(TObject *Sender)
{
    if (SaveDialog1->Execute()) {
        Edit2->Text = SaveDialog1->FileName;
        SaveDialog1->InitialDir = ExtractFilePath(Edit2->Text);
        }
    
}
//---------------------------------------------------------------------------
void __fastcall THPGLtoDXFForm::HelpButtonClick(TObject *Sender)
{
    AnsiString Message = "This program takes an HPGL-1 File, and translates drawing related information into a DXF file\n\nTo use the program select the input and output files then press the 'Translate' Button.\n\nThe output DXF file is in millimetres, and a scale of 0.025mm per HPGL unit is used.";
    Message += "\n\nThe commands translated are:-\n\tCI\tCircle, \n\tPU\tPen Up, \n\tPD\tPen Down, \n\tPA\tPen Absolute Position, \n\tPR\tPen Relative Position and \n\tSP\tSelect Pen. \nThese commands cover the HPGL-1 or PLT output from most programs. If you need more commands translated, contact " + TechnicalSupport + "\n\nPlease note:-\tHPGL-2 files are not translated.";
    Application->MessageBox(Message.c_str(),"Help on HPGL to DXF Translator Program",MB_OK | MB_ICONINFORMATION);

}
//---------------------------------------------------------------------------
void __fastcall THPGLtoDXFForm::CloseButtonClick(TObject *Sender)
{
    Close();    
}
//---------------------------------------------------------------------------
void __fastcall THPGLtoDXFForm::TranslateButtonClick(TObject *Sender)
{
    //Translaton Happens Here!
    if (!Edit1->Text.Length() || !Edit2->Text.Length()) {
        Application->MessageBox("Need two file names\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (Edit1->Text.LowerCase() == Edit2->Text.LowerCase()) {
        Application->MessageBox("Both file names the same\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (FileExists(Edit1->Text) && !FileExists(Edit2->Text)) {
        try {
            Screen->Cursor = crHourGlass;
            BrowseButton1->Enabled = false;
            BrowseButton2->Enabled = false;
            TranslateButton->Enabled = false;
            CloseButton->Enabled = false;
            HelpButton->Enabled = false;
            Edit1->Enabled = false;
            Edit2->Enabled = false;
            Refresh();

            HPGLFile = new TStringList();
            DXFFile = new TStringList();
            WriteInitialLines();
            Label3->Caption = "Opening File";
            Refresh();
            HPGLFile->LoadFromFile(Edit1->Text);
            Label3->Caption = "Sorting File";
            Refresh();

            Start.x = 0.0;
            Start.y = 0.0;
            Finish.x = 0.0;
            Finish.y = 0.0;
            PenStateDown = false;
            int counter = 0;
            AnsiString temp;
            TStringList *templist = new TStringList();

            //go through file & ensure one command per line
            while (counter < HPGLFile->Count) {
                temp = HPGLFile->Strings[counter].Trim();
                //third method
                while (counter < HPGLFile->Count - 1 && RightString(temp,1) != ";") {
                    counter++;
                    temp += HPGLFile->Strings[counter].Trim();
                    }
                /*
                //second method
                if (RightString(temp,1) != ";") {
                    //third method
                    temp += ";";
                    HPGLFile->Strings[counter] = temp;
                    }
                */
                /*
                if (temp.Pos(";") != temp.Length()) {
                    //it's assumed that there's more than one command per line
                    templist->Clear();
                    while (temp.Length()) {
                        templist->Add(ExtractWord(temp,";") + ";");
                        temp = DeleteWord(temp,";");
                        }
                    //all the commands are in templist
                    HPGLFile->Delete(counter);
                    for (int i = templist->Count - 1; i > -1; i--) {
                        HPGLFile->Insert(counter,templist->Strings[i]);
                        }
                    //counter += templist->Count;
                    }
                */
                //now to parse through line
                int start = 1;
                int end = 1;
                bool found;
                //templist->Clear();
                while (end < temp.Length()) {
                    //look for the command start
                    found = false;
                    AnsiString shortword;
                    while (start < temp.Length() && !found) {
                        shortword = temp.SubString(start,2);
                        if (shortword == "PU" || shortword == "PD"  || shortword == "PR" || shortword == "PA"  || shortword == "CI"  || shortword == "SP") {
                            start--;
                            found = true;
                            }
                        start++;
                        }
                    //now to look for the command end
                    end = start + 2;
                    if (found) {
                        found = false;
                        while (end < temp.Length() + 1 && !found) {
                            if (AnsiString(temp[end]) == ";") {
                                found = true;
                                end--;
                                } else {
                                if (AnyLettersPresent(AnsiString(temp[end]))) {
                                        end -= 2;
                                        found = true;
                                        }
                                /*
                                if (AnsiString(temp[end]) == "P" || AnsiString(temp[end]) == "C" || AnsiString(temp[end]) == "S") {
                                    shortword = temp.SubString(end,2);
                                    if (shortword == "PU" || shortword == "PD"  || shortword == "PR" || shortword == "PA"  || shortword == "CI"  || shortword == "SP") {
                                        end--;
                                        found = true;
                                        }
                                    }
                                */
                                }
                            end++;
                            }
                        if (found) {
                            //save the new line to templist
                            if (AnsiString(temp[end]) != ";") {
                                templist->Add(ReplaceChars(temp.SubString(start,end-start + 1)," ",",") + ";");
                                } else {
                                templist->Add(ReplaceChars(temp.SubString(start,end-start + 1)," ",","));
                                }
                            if (!(templist->Count % 1000)) {
                                Label3->Caption = "Sorted Line " + AnsiString(templist->Count);
                                Label3->Refresh();
                                }
                            start = end + 1;
                            }
                        }
                    }

                counter++;
                }

            //delete templist;
            try {
                //for debug
                //templist->SaveToFile(ChangeFileExt(Edit2->Text,".dxfhpg"));
                //end debug
                Label3->Caption = "Transferring sorted data";
                Refresh();
                HPGLFile->Clear();
                HPGLFile->Assign(templist);
                delete templist;
                Label3->Caption = "Parsing";
                Refresh();
                }
            catch (...) {
                Label3->Caption = "Error Transferring sorted data";
                Refresh();
                }


            //Now to do the work

            counter = 0;
            int filelength = HPGLFile->Count;

            while (counter < HPGLFile->Count) {
                if (!(counter % 1000)) {
                    Label3->Caption = "Line " + AnsiString(counter) + " of " + AnsiString(filelength);
                    if (!(counter %10000)) {
                        Refresh();
                        } else {
                        Label3->Refresh();
                        }
                    }
                temp = HPGLFile->Strings[counter].Trim();
                //AnsiString shortword = LeftString(temp,2);
                //temp = ChopOffFirstChars(temp,2);
                if (temp.Length() > 3) {
                    GetCoords(temp);
                    } else {
                    Nodes.Length = 0;
                    }
                if (temp.Pos("PA") == 1) PlotAbs();
                if (temp.Pos("PR") == 1) PlotRel();
                if (temp.Pos("PD") == 1) PlotDown();
                if (temp.Pos("PU") == 1) PlotUp();
                if (temp.Pos("SP") == 1) SetPen();
                if (temp.Pos("CI") == 1) Circle();
                counter++;
                }

            Label3->Caption = "Saving File";
            Refresh();

            WriteFinalLines();

            DXFFile->SaveToFile(Edit2->Text);

            Label3->Caption = "Clearing Memory";
            DXFFile->Clear();
            HPGLFile->Clear();
            Refresh();


            Label3->Caption = "Finished";
            Refresh();
            }

        __finally {
            BrowseButton1->Enabled = true;
            BrowseButton2->Enabled = true;
            TranslateButton->Enabled = true;
            CloseButton->Enabled = true;
            HelpButton->Enabled = true;
            Edit1->Enabled = true;
            Edit2->Enabled = true;
            Nodes.Length = 0;
            Refresh();
            Screen->Cursor = crDefault;
            }

        } else {
        if (FileExists(Edit1->Text)) {
            Application->MessageBox("An existing translated file will get overwritten.\n\nPlease delete file or change file name","Warning", MB_OK | MB_ICONERROR);
            } else {
            Application->MessageBox("The input file does not exist.\n\nPlease check","Warning", MB_OK | MB_ICONERROR);
            }
        Label3->Caption = "Translation Aborted";
        }

}
//---------------------------------------------------------------------------

void __fastcall THPGLtoDXFForm::WriteInitialLines(void)
{
    DXFFile->Add("999");
    DXFFile->Add("DXF File Produced by the EPS Logistics Technology Ltd HPGL Translator Program");
    DXFFile->Add("999");
    DXFFile->Add("Program Copyright EPS Logistics Technology Ltd");
    DXFFile->Add("999");
    DXFFile->Add("\tEPS Logistics Technology Ltd");
    DXFFile->Add("999");
    DXFFile->Add("\t152 Staplehurst Road");
    DXFFile->Add("999");
    DXFFile->Add("\tSittingbourne");
    DXFFile->Add("999");
    DXFFile->Add("\tKent");
    DXFFile->Add("999");
    DXFFile->Add("\tME10 1XS");
    DXFFile->Add("999");
    DXFFile->Add("Sales Contact:-\t" + Sales);
    DXFFile->Add("999");
    DXFFile->Add("Technical Support:-\t" + TechnicalSupport);
    DXFFile->Add(" 0");
    DXFFile->Add("SECTION");
    DXFFile->Add(" 2");
    DXFFile->Add("ENTITIES");
}
//---------------------------------------------------------------------------

void __fastcall THPGLtoDXFForm::WriteFinalLines(void)
{
    DXFFile->Add(" 0");
    DXFFile->Add("ENDSEC");
    DXFFile->Add(" 0");
    DXFFile->Add("EOF");
}
//---------------------------------------------------------------------------

void __fastcall THPGLtoDXFForm::WriteDXFLine(APoint Node1, APoint Node2)
{
    //actually writes the line to the output file
    DXFFile->Add(" 0");
    DXFFile->Add("LINE");
    DXFFile->Add(" 8");
    DXFFile->Add(Colour);
    DXFFile->Add("62");
    DXFFile->Add("7");
    //corner 1
    DXFFile->Add("10");
    DXFFile->Add(FloatToStrF(Node1.x,ffFixed,10,10));
    DXFFile->Add("20");
    DXFFile->Add(FloatToStrF(Node1.y,ffFixed,10,10));
    //corner 2
    DXFFile->Add("11");
    DXFFile->Add(FloatToStrF(Node2.x,ffFixed,10,10));
    DXFFile->Add("21");
    DXFFile->Add(FloatToStrF(Node2.y,ffFixed,10,10));

}
//---------------------------------------------------------------------------

void __fastcall THPGLtoDXFForm::PlotAbs(void)
{
        Start = Finish;
        for (int i = 0; i < Nodes.Length; i++) {
            Finish = Nodes[i];
            if (PenStateDown == true) WriteDXFLine(Start, Finish);
            Start = Finish;
            }
}
//---------------------------------------------------------------------------

void __fastcall THPGLtoDXFForm::GetCoords(AnsiString Command)
{
    Command = Command.SubString(3,Command.Length() - 2);
    Nodes.Length = 0;
    AnsiString temp;
    while (Command.Length()) {
        Nodes.Length++;
        if (Command.Pos(",")) {
            Nodes[Nodes.Length - 1].x = ToDoubleDef(ExtractWord(Command,","),0.0) * 0.025;
            Command = DeleteWord(Command,",");
            } else {
            Nodes[Nodes.Length - 1].x = ToDoubleDef(ExtractWord(Command,";"),0.0) * 0.025;
            Command = DeleteWord(Command,";");
            }
        if (Command.Pos(",")) {
            Nodes[Nodes.Length - 1].y = ToDoubleDef(ExtractWord(Command,","),0.0) * 0.025;
            Command = DeleteWord(Command,",");
            } else {
            Nodes[Nodes.Length - 1].y = ToDoubleDef(ExtractWord(Command,";"),0.0) * 0.025;
            Command = DeleteWord(Command,";");
            }
        }
}
//---------------------------------------------------------------------------

void __fastcall THPGLtoDXFForm::PlotRel(void)
{
        Start = Finish;
        for (int i = 0; i < Nodes.Length; i++) {
            Finish.x = Nodes[i].x + Start.x;
            Finish.y = Nodes[i].y + Start.y;
            if (PenStateDown == true) WriteDXFLine(Start, Finish);
            Start = Finish;
            }
}
//---------------------------------------------------------------------------

void __fastcall THPGLtoDXFForm::PlotDown(void)
{
        PenStateDown = true;
        PlotAbs();
}
//---------------------------------------------------------------------------

void __fastcall THPGLtoDXFForm::PlotUp(void)
{
        PenStateDown = false;
        PlotAbs();
}
//---------------------------------------------------------------------------

void __fastcall THPGLtoDXFForm::SetPen(void)
{
    if (Nodes.Length) {
        Colour = AnsiString(Nodes[0].x);
        }
}
//---------------------------------------------------------------------------

void __fastcall THPGLtoDXFForm::Circle(void)
{
    if (Nodes.Length) {
        //actually writes the line to the output file
        DXFFile->Add(" 0");
        DXFFile->Add("CIRCLE");
        DXFFile->Add(" 8");
        DXFFile->Add(Colour);
        DXFFile->Add("62");
        DXFFile->Add("7");
        //centre
        DXFFile->Add("10");
        DXFFile->Add(FloatToStrF(Finish.x,ffFixed,10,10));
        DXFFile->Add("20");
        DXFFile->Add(FloatToStrF(Finish.y,ffFixed,10,10));
        //radius
        DXFFile->Add("40");
        DXFFile->Add(FloatToStrF(Nodes[0].x,ffFixed,10,10));
        }
}
//---------------------------------------------------------------------------



